@@ -16,6 +16,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-from .core import Client, Project, Pipeline, Result, Terminology
+from .core import Client, Project, Pipeline, Result, Terminology, DocumentCollection
 
-__all__ = ["Client", "Project", "Pipeline", "Result", "Terminology"]
+__all__ = ["Client", "Project", "Pipeline", "Result", "Terminology", "DocumentCollection"]
